@@ -26,7 +26,7 @@ FindFile::~FindFile()
 
 void FindFile::SetMask(const wchar *Mask)
 {
-  wcscpy(FindMask,Mask);
+  wcsncpyz(FindMask,Mask,ASIZE(FindMask));
   FirstCall=true;
 }
 
@@ -52,7 +52,7 @@ bool FindFile::Next(FindData *fd,bool GetSymLink)
     wcsncpyz(DirName,FindMask,ASIZE(DirName));
     RemoveNameFromPath(DirName);
     if (*DirName==0)
-      wcscpy(DirName,L".");
+      wcsncpyz(DirName,L".",ASIZE(DirName));
     char DirNameA[NM];
     WideToChar(DirName,DirNameA,ASIZE(DirNameA));
     if ((dirp=opendir(DirNameA))==NULL)
@@ -75,20 +75,20 @@ bool FindFile::Next(FindData *fd,bool GetSymLink)
     if (CmpName(FindMask,Name,MATCH_NAMES))
     {
       wchar FullName[NM];
-      wcscpy(FullName,FindMask);
+      wcsncpyz(FullName,FindMask,ASIZE(FullName));
       *PointToName(FullName)=0;
       if (wcslen(FullName)+wcslen(Name)>=ASIZE(FullName)-1)
       {
         uiMsg(UIERROR_PATHTOOLONG,FullName,L"",Name);
         return false;
       }
-      wcscat(FullName,Name);
+      wcsncatz(FullName,Name,ASIZE(FullName));
       if (!FastFind(FullName,fd,GetSymLink))
       {
         ErrHandler.OpenErrorMsg(FullName);
         continue;
       }
-      wcscpy(fd->Name,FullName);
+      wcsncpyz(fd->Name,FullName,ASIZE(fd->Name));
       break;
     }
   }
@@ -179,9 +179,14 @@ HANDLE FindFile::Win32Find(HANDLE hFind,const wchar *Mask,FindData *fd)
     if (hFind==INVALID_HANDLE_VALUE)
     {
       int SysErr=GetLastError();
-      fd->Error=(SysErr!=ERROR_FILE_NOT_FOUND &&
-                 SysErr!=ERROR_PATH_NOT_FOUND &&
-                 SysErr!=ERROR_NO_MORE_FILES);
+      // We must not issue an error for "file not found" and "path not found",
+      // because it is normal to not find anything for wildcard mask when
+      // archiving. Also searching for non-existent file is normal in some
+      // other modules, like WinRAR scanning for winrar_theme_description.txt
+      // to check if any themes are available.
+      fd->Error=SysErr!=ERROR_FILE_NOT_FOUND && 
+                SysErr!=ERROR_PATH_NOT_FOUND &&
+                SysErr!=ERROR_NO_MORE_FILES;
     }
   }
   else
