@@ -127,11 +127,11 @@
 #define   MCHelpSwT          L"\n  t             Test files after archiving"
 #define   MCHelpSwTK         L"\n  tk            Keep original archive time"
 #define   MCHelpSwTL         L"\n  tl            Set archive time to latest file"
-#define   MCHelpSwTN         L"\n  tn<time>      Process files newer than <time>"
-#define   MCHelpSwTO         L"\n  to<time>      Process files older than <time>"
-#define   MCHelpSwTA         L"\n  ta<date>      Process files modified after <date> in YYYYMMDDHHMMSS format"
-#define   MCHelpSwTB         L"\n  tb<date>      Process files modified before <date> in YYYYMMDDHHMMSS format"
-#define   MCHelpSwTS         L"\n  ts[m|c|a]     Save or restore file time (modification, creation, access)"
+#define   MCHelpSwTN         L"\n  tn[m,c,a]<t>  Process files newer than <t> time"
+#define   MCHelpSwTO         L"\n  to[m,c,a]<t>  Process files older than <t> time"
+#define   MCHelpSwTA         L"\n  ta[m,c,a]<d>  Process files modified after <d> YYYYMMDDHHMMSS date"
+#define   MCHelpSwTB         L"\n  tb[m,c,a]<d>  Process files modified before <d> YYYYMMDDHHMMSS date"
+#define   MCHelpSwTS         L"\n  ts[m,c,a]     Save or restore file time (modification, creation, access)"
 #define   MCHelpSwU          L"\n  u             Update files"
 #define   MCHelpSwV          L"\n  v             Create volumes with size autodetection or list all volumes"
 #define   MCHelpSwVUnr       L"\n  v             List all volumes"
@@ -354,7 +354,7 @@
 #define   MCannotDelete      L"\nCannot delete %s"
 #define   MRecycleFailed     L"\nCannot move some files and folders to Recycle Bin"
 #define   MCalcCRC           L"\nCalculating the checksum"
-#define   MTooLargeSFXArc    L"\nWARNING: Too large SFX archive. Windows cannot run the executable file exceeding 4 GB."
+#define   MTooLargeSFXArc    L"\nToo large SFX archive. Windows cannot run the executable file exceeding 4 GB."
 #define   MCalcCRCAllVol     L"\nCalculating checksums of all volumes."
 #define   MNotEnoughDisk     L"\nERROR: Not enough disk space for %s."
 #define   MNewerRAR          L"\nYou may need a newer version of RAR."
@@ -375,6 +375,7 @@
 #define   MCopyingData       L"\nCopying data"
 #define   MErrCreateLnkS     L"\nCannot create symbolic link %s"
 #define   MErrCreateLnkH     L"\nCannot create hard link %s"
+#define   MErrLnkTarget      L"\nYou need to unpack the link target first"
 #define   MNeedAdmin         L"\nYou may need to run RAR as administrator"
 #define   MDictOutMem        L"\nNot enough memory for %d MB compression dictionary, changed to %d MB."
 #define   MUseSmalllerDict   L"\nPlease use a smaller compression dictionary."
